//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Device/InstrumentCollection.cpp
//! @brief     Implement class InstrumentCollection
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Device/InstrumentCollection.h"
#include "GUI/Model/Device/InstrumentItems.h"
#include "GUI/Util/String.h"
#include <QUuid>

namespace {
namespace Tag {

const QString Instrument("Instrument");

} // namespace Tag
} // namespace

InstrumentCollection::~InstrumentCollection() {}

void InstrumentCollection::clear()
{
    m_instruments.clear();
}

void InstrumentCollection::writeTo(QXmlStreamWriter* w) const
{
    XML::writeAttribute(w, XML::Attrib::version, uint(1));

    // instruments
    for (const auto& sel : m_instruments) {
        w->writeStartElement(Tag::Instrument);
        sel.writeTo(w);
        w->writeEndElement();
    }
}

void InstrumentCollection::readFrom(QXmlStreamReader* r)
{
    m_instruments.clear();

    const uint version = XML::readUIntAttribute(r, XML::Attrib::version);
    Q_UNUSED(version)

    while (r->readNextStartElement()) {
        QString tag = r->name().toString();

        // instrument
        if (tag == Tag::Instrument) {
            addEmptyInstrumentSelection().readFrom(r);
            XML::gotoEndElementOfTag(r, tag);

        } else
            r->skipCurrentElement();
    }
}

void InstrumentCollection::emplace_back(InstrumentItem* item)
{
    addEmptyInstrumentSelection().setCurrentItem(item);
}

InstrumentItem* InstrumentCollection::insertCopy(const InstrumentItem& source)
{
    auto* copy = source.createCopy();
    copy->setId(QUuid::createUuid().toString());
    emplace_back(copy);
    return copy;
}

QVector<InstrumentItem*> InstrumentCollection::collectedItems() const
{
    QVector<InstrumentItem*> output;
    for (const auto& sel : m_instruments)
        output.append(sel.currentItem());
    return output;
}

QStringList InstrumentCollection::instrumentNames() const
{
    QStringList existingNames;
    for (const auto* item : collectedItems())
        existingNames << item->instrumentName();
    return existingNames;
}

SelectionProperty<InstrumentItemCatalog>& InstrumentCollection::addEmptyInstrumentSelection()
{
    SelectionProperty<InstrumentItemCatalog> newInstrumentSelection;
    m_instruments.push_back(std::move(newInstrumentSelection));
    return m_instruments.back();
}

QString InstrumentCollection::suggestInstrumentName(const QString& baseName) const
{
    return GUI::Util::String::suggestName(instrumentNames(), baseName);
}

QVector<InstrumentItem*> InstrumentCollection::instrumentItems(
    const std::function<bool(const InstrumentItem*)>& accept) const
{
    QVector<InstrumentItem*> result;
    for (auto* p : collectedItems())
        if (accept(p))
            result << p;

    return result;
}

QVector<Instrument2DItem*> InstrumentCollection::instrument2DItems() const
{
    QVector<Instrument2DItem*> result;
    for (auto* p : collectedItems())
        if (auto* p2D = dynamic_cast<Instrument2DItem*>(p))
            result << p2D;

    return result;
}

InstrumentItem* InstrumentCollection::findInstrumentById(const QString& instrumentId) const
{
    for (auto* instrument : collectedItems())
        if (instrument->id() == instrumentId)
            return instrument;

    return nullptr;
}

bool InstrumentCollection::instrumentExists(const QString& instrumentId) const
{
    return findInstrumentById(instrumentId) != nullptr;
}

void InstrumentCollection::removeInstrument(InstrumentItem* instrument)
{
    for (size_t i = 0; i < m_instruments.size(); i++)
        if (m_instruments[i].currentItem() == instrument)
            m_instruments.erase(m_instruments.begin() + i);
}
